// lscp_client_test.c

#include "lscp_client.h"


#define SERVER_PORT 8888

#if defined(WIN32)
static WSADATA _wsaData;
#endif

////////////////////////////////////////////////////////////////////////

lscp_status_t client_callback ( lscp_client_t *pClient, const char *pchBuffer, int cchBuffer, void *pvData )
{
    lscp_status_t ret = LSCP_OK;

    char *pszBuffer = (char *) malloc(cchBuffer + 1);
    if (pszBuffer) {
        memcpy(pszBuffer, pchBuffer, cchBuffer);
        pszBuffer[cchBuffer] = (char) 0;
        printf("client_callback: [%s]\n", pszBuffer);
        free(pszBuffer);
    }
    else ret = LSCP_FAILED;

    return ret;
}

////////////////////////////////////////////////////////////////////////

void client_usage (void)
{
    fputs("\n  Available client commands: exit, subscribe, unsubscribe", stdout);
    fputs("\n  (all else are sent verbatim to server)\n\n", stdout);
}

void client_prompt (void)
{
    fputs("lscp_client> ", stdout);
}

int main (int argc, char *argv[] )
{
    lscp_client_t *pClient;
    char *pszHost = "localhost";
    char  szLine[200];
    int  cchLine;

#if defined(WIN32)
    if (WSAStartup(MAKEWORD(1, 1), &_wsaData)) {
        fprintf(stderr, "lscp_client: WSAStartup failed.\n");
        return -1;
    }
#endif

    if (argc > 1)
        pszHost = argv[1];

    pClient = lscp_client_create(pszHost, SERVER_PORT, client_callback, NULL);
    if (pClient == NULL)
        return -1;

    client_usage();
    client_prompt();

    while (fgets(szLine, sizeof(szLine) - 1, stdin)) {

        cchLine = strlen(szLine);
        while (cchLine > 0 && (szLine[cchLine - 1] == '\n' || szLine[cchLine - 1] == '\r'))
            cchLine--;
        szLine[cchLine] = '\0';

        if (strcmp(szLine, "exit") == 0)
            break;
        else
        if (strcmp(szLine, "subscribe") == 0)
            lscp_client_subscribe(pClient);
        else
        if (strcmp(szLine, "unsubscribe") == 0)
            lscp_client_unsubscribe(pClient);
        else
        if (cchLine > 0) {
            cchLine = sizeof(szLine) - 1;
            if (lscp_client_call(pClient, szLine, strlen(szLine), szLine, &cchLine) == LSCP_OK) {
                szLine[cchLine] = (char) 0;
                fputs(szLine, stdout);
                fputc('\n', stdout);
            }
        }
        else client_usage();
        
        client_prompt();
    }

    lscp_client_destroy(pClient);

#if defined(WIN32)
    WSACleanup();
#endif

    return 0;
}

// end of lscp_client_test.c

